-- classes.e
-- do not modify this file
-- add separate files for user classes
-- define the class files in includes.e

include myLibs\myDebug.e

-- a class is a template
-- it is refered to by a name
-- it contains values and methods (functions)
-- to use a class, you must create entities based on the class

-- to manage classes and entities, we use 2 sequences
--
-- the first one is named classes. It lists classes.
--   each class item consists of 2 sequences: {name, methods}
--     name is the name of the class
--     methods is a sequence listing all methods in form of pairs {method name, routine id}
--   there is only one method to manage classes
--     procedure defineClass(name, methods) to define a class
--
-- the second one is named entities. It lists entities.
--   each entity item consists of 3 sequences: {name, class, values}
--     name is the name of the entity
--     class is the base class name
--     values is a sequence listing all values contained in the entity
--   there are 3 methods to manage entities
--     procedure createEntity(sequence name, sequence class, sequence values) to create an entity
--       name is the entity's name
--       class is the base class' name
--       values is a sequence of initial values of all contained variables
--     procedure removeEntity(sequence name) to remove an entity
--       name is the name of the entity
--     function method(sequence entity, sequence name, sequence params) to call a method (function) within the entity
--       entity is the name of the entity
--       name is the method's name
--       params is a sequence containing all the method parameters
--       the function returns the result of the method

------------------------------------------------------------------------------
-- INTERNAL CONSTANTS
------------------------------------------------------------------------------

constant CLASS_NAME =1, CLASS_METHODS=2
constant METHOD_NAME=1, METHOD_ID=2

------------------------------------------------------------------------------
-- GLOBAL CONSTANTS
------------------------------------------------------------------------------

global constant ENTITY_ID=1, ENTITY_CLASS=2, ENTITY_VALUES=3

------------------------------------------------------------------------------
-- INTERNAL VARIABLES
------------------------------------------------------------------------------

sequence classes
classes = {}

integer lastID
lastID = 0

global sequence entities
entities = {}

------------------------------------------------------------------------------
-- INTERNAL FUNCTIONS
------------------------------------------------------------------------------

-- returns first item from sheet sht with value val in field fld
function findItem(sequence sht, integer fld, object val)
  if fld = 0 then return 0 end if
  for i = 1 to length(sht) do
    if compare(sht[i][fld], val) = 0 then return i end if
  end for
  return 0
end function

------------------------------------------------------------------------------

function findMethod(sequence class, sequence name)
  integer cl, meth

  cl = findItem(classes, CLASS_NAME, class)
  if cl = 0 then
    analyzeSequence(classes, "classes", f_debug)
    printf(f_debug, "class = %s, cl = %d\n", {name, cl})
    printf(f_debug, "class %s not found\n", {class})
    return -1
  end if
  meth = findItem(classes[cl][CLASS_METHODS], METHOD_NAME, name)
  if meth > 0 then
    return classes[cl][CLASS_METHODS][meth][METHOD_ID]
  end if
  return 0
end function

------------------------------------------------------------------------------

function findEntity(integer en)
  return findItem(entities, ENTITY_ID, en)
end function

------------------------------------------------------------------------------
-- GLOBAL FUNCTIONS
------------------------------------------------------------------------------

-- define a class
global procedure defineClass(sequence name, sequence methods)
  sequence s

  s = {}
  for i = 1 to length(methods) do
    s = append(s, methods[i])
  end for
  classes = append(classes, {name, s})
end procedure

------------------------------------------------------------------------------

-- create an entity of the class and preset its values
-- doesn't care which type the value is: you should know
global function createEntity(sequence class, sequence values)
  lastID += 1
  entities = append(entities, {lastID, class, values})
  return lastID
end function

------------------------------------------------------------------------------

-- remove en entity
global procedure removeEntity(integer entity)
  integer en

  en = findEntity(entity)
  if en = 0 then
    analyzeSequence(entities, "entities", f_debug)
    printf(f_debug, "entity %s not found\n", {entity})
    return
  end if
  if en = 1 then
    entities = entities[2..$]
  elsif en = length(entities) then
    entities = entities[1..$-1]
  else
    entities = entities[1..en-1] & entities[en+1..$]
  end if
end procedure

------------------------------------------------------------------------------

-- call a method (function) within an entity
global function callMethod(integer entity, sequence name, sequence params)
  integer meth, en
  sequence cl

  en = findEntity(entity)
  if en = 0 then
    analyzeSequence(entities, "entities", f_debug)
    printf(f_debug, "entity %s not found\n", {entity})
    return -1
  end if
  cl = entities[en][ENTITY_CLASS]
  meth = findMethod(cl, name)
  if meth = -1 then
    analyzeSequence(classes, "classes", f_debug)
    analyzeSequence(entities, "entities", f_debug)
    printf(f_debug, "cl = %d, meth = %d\n", {cl, meth})
    printf(f_debug, "method %s not found\n", {name})
    return -1
  end if
  return call_func(meth, {en, params})
end function

------------------------------------------------------------------------------

