-- dbedit.e

include classes.e
include myLibs\myDebug.e
include myLibs\myFile.e

------------------------------------------------------------------------------
-- USER CONSTANTS AND VARIABLES
--
-- in this section, you may put your own constants and variables

------------------------------------------------------------------------------

global constant
  EDT_STRING = 1, EDT_INTEGER = 2, EDT_FLOAT = 3,
  EDT_ID = 1, EDT_TYPE = 2, EDT_TEXT = 3, EDT_TABLE = 4, EDT_FIELD = 5, EDT_UPDATED = 6

------------------------------------------------------------------------------
-- TDBEdit

--   sequence {
--             id,    -- underlying EditText control ID
--             text,  -- text of the control
--             type,  -- type of the control
--             table, -- linked table
--             updated    -- flag set when control is modified
--            )
--   function editCreate(integer en, sequence params)
--   function editGetControlID(integer en, sequence params)
--   function editClean(integer en, sequence params)
--   function editUpdate(integer en, sequence params)
--   function editModified(integer en, sequence params)
--   function editShow(integer en, sequence params)
--   function editHide(integer en, sequence params)

------------------------------------------------------------------------------

-- params = {id,text,type,table,updated}
global function editCreate(sequence params)
  integer en

  en = createEntity("TDBEdit",
      {
        "id",    -- underlying EditText control ID
        "type",  -- type of the control
        "text",  -- text of the control
        "table", -- linked table
        "updated"    -- flag set when control is modified
      }
    )
  entities[en][ENTITY_VALUES] = params
  void = callMethod(params[EDT_TABLE], "tableSetLink",
   {params[EDT_ID],EditText,params[EDT_FIELD]})
  return en
end function

------------------------------------------------------------------------------

-- params = {}
global  function editGetControlID(integer en, sequence params)
  return entities[en][ENTITY_VALUES][EDT_ID]
end function

------------------------------------------------------------------------------

-- empty control text
-- params = {}
global function editClean(integer en, sequence params)
  setText(entities[en][ENTITY_VALUES][EDT_ID], "")
  entities[en][ENTITY_VALUES][EDT_TEXT] = ""
  return 0
end function

------------------------------------------------------------------------------

-- close Table
-- params = {{text}}
global function editUpdate(integer en, sequence params)
  sequence s, txt

  s = entities[en][ENTITY_VALUES]
  if s[EDT_TYPE] = EDT_STRING then
    txt = params[1]
  elsif s[EDT_TYPE] = EDT_INTEGER then
    txt = sprintf("%d", params[1])
  elsif s[EDT_TYPE] = EDT_FLOAT then
    txt = sprintf("%f", params[1])
  end if
  setText(s[EDT_ID], txt)
  s[EDT_TEXT] = txt
  entities[en][ENTITY_VALUES] = s
  return 0
end function

------------------------------------------------------------------------------

-- check if control was modified
-- params = {}
global function editModified(integer en, sequence params)
  return entities[en][ENTITY_VALUES][EDT_UPDATED]
end function

------------------------------------------------------------------------------

-- show control
-- params = {}
global function editShow(integer en, sequence params)
  setVisible(entities[en][ENTITY_VALUES][EDT_ID], w32True)
  return 0
end function

------------------------------------------------------------------------------

-- hide control
-- params = {}
global function editHide(integer en, sequence params)
  setVisible(entities[en][ENTITY_VALUES][EDT_ID], w32False)
  return 0
end function

------------------------------------------------------------------------------

  defineClass("TDBEdit", {
     {"editCreate", routine_id("editCreate")},
     {"editGetControlID", routine_id("editGetControlID")},
     {"editClean", routine_id("editClean")},
     {"editUpdate", routine_id("editUpdate")},
     {"editModified", routine_id("editModified")},
     {"editShow", routine_id("editShow")},
     {"editHide", routine_id("editHide")}
    }
  )

