-- table.e

include classes.e
include myLibs\myDebug.e
include myLibs\myFile.e
include myLibs\mySeq.e

------------------------------------------------------------------------------
-- USER CONSTANTS AND VARIABLES
--
-- in this section, you may put your own constants and variables

------------------------------------------------------------------------------

global constant
  TBL_FILENAME = 1, TBL_FIELDS = 2, TBL_DATA = 3,
  TBL_LINKS = 4, TBL_STATUS = 5, TBL_POINTER = 6,
  TBL_UPDATED = 7,
  LNK_ID = 1, LNK_TYPE = 2, LNK_FIELD = 3

------------------------------------------------------------------------------
-- TTable

--   sequence {
--             filename,  -- name of the table
--             fields,    -- name of each field
--             data,      -- table content
--             links,     -- sequence of components ids to update when table is modified
--             status,    -- status of the table: open or closed
--             pointer,   -- current record
--             updated    -- flag set when table is modified
--            )
--   function tableCreate(integer en, sequence params)
--   function tableOpen(integer en, sequence params)
--   function tableClose(integer en, sequence params)
--   function tableSetLink(integer en, sequence params)
--   function tableGetLinks(integer en, sequence params)
--   function tableRefreshLinks(integer en, sequence params)
--   function tableSetPointer(integer en, sequence params)
--   function tableGetPointer(integer en, sequence params)
--   function tableGetFields(integer en, sequence params)
--   function tableGetLength(integer en, sequence params)
--   function tableGoFirst(integer en, sequence params)
--   function tableGoPrevious(integer en, sequence params)
--   function tableGoNext(integer en, sequence params)
--   function tableGoLast(integer en, sequence params)
--   function tableUpdate(integer en, sequence params)
--   function tableModified(integer en, sequence params)
--   function tableGetStatus(integer en, sequence params)
--   function tableAddRecord(integer en, sequence params)
--   function tableInsertRecord(integer en, sequence params)
--   function tableModifyRecord(integer en, sequence params)
--   function tableDeleteRecord(integer en, sequence params)

------------------------------------------------------------------------------

-- create Table
-- params = {filename,fields,data,links,status,pointer,updated}
global function tableCreate(sequence params)
  integer en
  
  en = createEntity("TTable",
      {
        "filename", -- name of the database file
        "fields",   -- name of each field of the table
        "length",   -- # records
        "links",    -- list of components ids to update when table is modified
        "status",   -- open(1) or closed (0)
        "pointer",  -- current record
        "updated"   -- table has been modified (1) or not (0)
      }
    )
  entities[en][ENTITY_VALUES] = params
  return en
end function

------------------------------------------------------------------------------

-- open Table
-- params = {}
global function tableOpen(integer en, sequence params)
  sequence s
  
  s = entities[en][ENTITY_VALUES]
  s[TBL_DATA] = readCSV(s[TBL_FILENAME])
  s[TBL_STATUS] = 1
  s[TBL_POINTER] = 1
  s[TBL_UPDATED] = 0
  entities[en][ENTITY_VALUES] = s
  return 0
end function

------------------------------------------------------------------------------

-- close Table
-- params = {}
global function tableClose(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  saveCSV(entities[en][ENTITY_VALUES][TBL_FILENAME], entities[en][ENTITY_VALUES][TBL_DATA])
  s[TBL_DATA] = {}
  s[TBL_STATUS] = 0
  s[TBL_POINTER] = 0
  s[TBL_UPDATED] = 0
  entities[en][ENTITY_VALUES] = s
  return 0
end function

------------------------------------------------------------------------------

-- set Table link
-- (component to update when table is modified)
-- params = {component id}
global function tableSetLink(integer en, sequence params)
  entities[en][ENTITY_VALUES][TBL_LINKS] = append(entities[en][ENTITY_VALUES][TBL_LINKS], params)
  return 0
end function

------------------------------------------------------------------------------

-- get Table links
-- (list of components to update when table is modified)
-- params = {}
global function tableGetLinks(integer en, sequence params)
  return entities[en][ENTITY_VALUES][TBL_LINKS]
end function

------------------------------------------------------------------------------

-- update linked components
-- params = {}
global function tableRefreshLinks(integer en, sequence params)
  sequence s
  integer rec, fld

  s = entities[en][ENTITY_VALUES]
--  analyzeSequence(s, "s", f_debug)
  rec = s[TBL_POINTER]
--  printf(f_debug, "rec = %d\n", rec)
--  analyzeSequence(s[TBL_FIELDS], "s[TBL_FIELDS]", f_debug)
--  analyzeSequence(s[TBL_LINKS], "s[TBL_LINKS]", f_debug)
  for i = 1 to length(s[TBL_LINKS]) do
    fld = find(s[TBL_LINKS][i][LNK_FIELD], s[TBL_FIELDS])
--    printf(f_debug, "fld = %d\n", fld)
    if s[TBL_LINKS][i][LNK_TYPE] = EditText then
      setText(s[TBL_LINKS][i][LNK_ID], s[TBL_DATA][rec][fld])
    end if
  end for
  return 0
end function

------------------------------------------------------------------------------

-- go to specified record num
-- params = {record num}
global function tableSetPointer(integer en, sequence params)
  entities[en][ENTITY_VALUES][TBL_POINTER] = params[1]
  void = tableRefreshLinks(en, {})
  return 0
end function

------------------------------------------------------------------------------

-- get Table Pointer
-- params = {}
global function tableGetPointer(integer en, sequence params)
  return entities[en][ENTITY_VALUES][TBL_POINTER]
end function

------------------------------------------------------------------------------

-- get Table Fields
-- params = {}
global function tableGetFields(integer en, sequence params)
  return entities[en][ENTITY_VALUES][TBL_FIELDS]
end function

-----------------------------------------------------------------------------

-- get Table Size (# records)
-- params = {}
global function tableGetLength(integer en, sequence params)
  return length(entities[en][ENTITY_VALUES][TBL_DATA])
end function

------------------------------------------------------------------------------

-- go to first record
-- params = {}
global function tableGoFirst(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  if length(s[TBL_DATA])=0 then return 1 end if
  void = tableSetPointer(en, {1})
  return 0
end function

------------------------------------------------------------------------------

-- go to previous record
-- params = {}
global function tableGoPrevious(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  if length(s[TBL_DATA])=0 then return 1 end if
  if s[TBL_POINTER]=1 then return 0 end if
  void = tableSetPointer(en, {s[TBL_POINTER] - 1})
  return 0
end function

------------------------------------------------------------------------------

-- go to next record
-- params = {}
global function tableGoNext(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  if length(s[TBL_DATA])=0 then return 1 end if
  if s[TBL_POINTER]=length(s[TBL_DATA]) then return 0 end if
  void = tableSetPointer(en, {s[TBL_POINTER] + 1})
  return 0
end function

------------------------------------------------------------------------------

-- go to last record
-- params = {}
global function tableGoLast(integer en, sequence params)
  sequence s
  object void

  s = entities[en][ENTITY_VALUES]
  if length(s[TBL_DATA])=0 then return 1 end if
  void = tableSetPointer(en, {length(s[TBL_DATA])})
  return 0
end function

------------------------------------------------------------------------------

-- update current record with content of linked components
-- params = {}
global function tableUpdate(integer en, sequence params)
  sequence s
  integer rec, fld

  s = entities[en][ENTITY_VALUES]
  rec = s[TBL_POINTER]
  for i = 1 to length(s[TBL_LINKS]) do
    fld = find(s[TBL_LINKS][i][LNK_FIELD], s[TBL_FIELDS])
    if s[TBL_LINKS][i][LNK_TYPE] = EditText then
      s[TBL_DATA][rec][fld] = getText(s[TBL_LINKS][i][LNK_ID])
    end if
  end for
  entities[en][ENTITY_VALUES] = s
  return 0
end function

------------------------------------------------------------------------------

-- check if Table was modified
-- params = {}
global function tableModified(integer en, sequence params)
  return entities[en][ENTITY_VALUES][TBL_UPDATED]
end function

------------------------------------------------------------------------------

-- get Table status: open (1) or closed (0)
-- params = {}
global function tableGetStatus(integer en, sequence params)
  return entities[en][ENTITY_VALUES][TBL_STATUS]
end function

------------------------------------------------------------------------------

-- add a record at end of Table
-- params = CSV record to add
global function tableAddRecord(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  s[TBL_DATA] = append(s[TBL_DATA], params)
  entities[en][ENTITY_VALUES] = s
  void = tableGoLast(en, {})
  return 0
end function

------------------------------------------------------------------------------

-- insert a record at current position of Table
-- params = CSV record to insert
global function tableInsertRecord(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  s[TBL_DATA] = insertInSequence({params}, s[TBL_DATA], s[TBL_POINTER])
  entities[en][ENTITY_VALUES] = s
  void = tableRefreshLinks(en, {})
  return 0
end function

------------------------------------------------------------------------------

-- modify current record
-- params = modified CSV record to update
global function tableModifyRecord(integer en, sequence params)
  sequence s
  integer rec
  
  s = entities[en][ENTITY_VALUES]
  rec = s[TBL_POINTER]
  s[TBL_DATA][rec] = params
  entities[en][ENTITY_VALUES] = s
  void = tableRefreshLinks(en, {})
  return 0
end function

-----------------------------------------------------------------------------

-- delete current record
-- params = {}
global function tableDeleteRecord(integer en, sequence params)
  sequence s

  s = entities[en][ENTITY_VALUES]
  s[TBL_DATA] = removeFromSequence(s[TBL_DATA], s[TBL_POINTER])
  entities[en][ENTITY_VALUES] = s
  void = tableRefreshLinks(en, {})
  return 0
end function

------------------------------------------------------------------------------

  defineClass("TTable", {
     {"tableCreate", routine_id("tableCreate")},
     {"tableOpen", routine_id("tableOpen")},
     {"tableClose", routine_id("tableClose")},
     {"tableSetLink", routine_id("tableSetLink")},
     {"tableGetLinks", routine_id("tableGetLinks")},
     {"tableRefreshLinks", routine_id("tableRefreshLinks")},
     {"tableSetPointer", routine_id("tableSetPointer")},
     {"tableGetPointer", routine_id("tableGetPointer")},
     {"tableGetFields", routine_id("tableGetFields")},
     {"tableGetLength", routine_id("tableGetLength")},
     {"tableGoFirst", routine_id("tableGoFirst")},
     {"tableGoPrevious", routine_id("tableGoPrevious")},
     {"tableGoNext", routine_id("tableGoNext")},
     {"tableGoLast", routine_id("tableGoLast")},
     {"tableUpdate", routine_id("tableUpdate")},
     {"tableModified", routine_id("tableModified")},
     {"tableStatus", routine_id("tableStatus")},
     {"tableAddRecord", routine_id("tableAddRecord")},
     {"tableInsertRecord", routine_id("tableInsertRecord")},
     {"tableModifyRecord", routine_id("tableModifyRecord")},
     {"tableDeleteRecord", routine_id("tableDeleteRecord")}
    }
  )

